@@ -36,6 +36,11 @@ is.discrete <- function(x) {
   if (length(x) > 0 || is_blank(x)) x else y
 }
 
+"%()%" <- function(x, y) {
+  if (is.function(x)) return(x())
+  y
+}
+
 # kind of like %||%, but only respects user-defined defaults
 # (instead of defaults provided in the build step)
 "%|D|%" <- function(x, y) {
@@ -50,7 +55,7 @@ is.default <- function(x) {
 }
 
 default <- function(x) {
-  structure(x, class = "plotly_default")
+  prefix_class(x %||% list(), "plotly_default")
 }
 
 compact <- function(x) {
@@ -69,6 +74,19 @@ to_milliseconds <- function(x) {
   x
 }
 
+# apply a function to x, retaining class and "special" plotly attributes
+retain <- function(x, f = identity) {
+  y <- structure(f(x), class = oldClass(x))
+  attrs <- attributes(x)
+  # TODO: do we set any other "special" attributes internally 
+  # (grepping "structure(" suggests no)
+  attrs <- attrs[names(attrs) %in% "apiSrc"]
+  if (length(attrs)) {
+    attributes(y) <- attrs
+  }
+  y
+}
+
 deparse2 <- function(x) {
   if (is.null(x) || !is.language(x)) return(NULL)
   sub("^~", "", paste(deparse(x, 500L), collapse = ""))
@@ -89,10 +107,11 @@ getLevels <- function(x) {
 tryNULL <- function(expr) tryCatch(expr, error = function(e) NULL)
 
 # Don't attempt to do "tidy" data training on these trace types
+# Note that non-tidy traces expect/anticipate data_array's of varying lengths
 is_tidy <- function(trace) {
   type <- trace[["type"]] %||% "scatter"
   !type %in% c(
-    "mesh3d", "heatmap", "histogram2d", 
+    "mesh3d", "heatmap", "histogram2d", "isosurface",
     "histogram2dcontour", "contour", "surface"
   )
 }
@@ -105,32 +124,22 @@ has_group <- function(trace) {
 
 # currently implemented non-positional scales in plot_ly()
 npscales <- function() {
-  c("color", "symbol", "linetype", "size", "split")
+  c("color", "stroke", "symbol", "linetype", "size", "span", "split")
 }
 
-# copied from https://github.com/plotly/plotly.js/blob/master/src/components/color/attributes.js
-traceColorDefaults <- function() {
-  c('#1f77b4', '#ff7f0e', '#2ca02c', '#d62728', '#9467bd', 
-    '#8c564b', '#e377c2', '#7f7f7f', '#bcbd22', '#17becf')
+colorway <- function(p = NULL) {
+  colway <- p$x$layout$colorway %||% Schema$layout$layoutAttributes$colorway$dflt
+  lapply(as.list(colway), function(x) structure(x, class = "colorway"))
 }
 
 # column name for crosstalk key
 # TODO: make this more unique?
 crosstalk_key <- function() ".crossTalkKey"
 
-# modifyList turns elements that are data.frames into lists
-# which changes the behavior of toJSON
-as_df <- function(x) {
-  if (is.null(x) || is.matrix(x)) return(x)
-  if (is.list(x) && !is.data.frame(x)) {
-    setNames(as.data.frame(x), NULL)
-  }
-}
-
 # arrange data if the vars exist, don't throw error if they don't
 arrange_safe <- function(data, vars) {
   vars <- vars[vars %in% names(data)]
-  if (length(vars)) dplyr::arrange_(data, .dots = vars) else data
+  if (length(vars)) dplyr::arrange(data, !!!rlang::syms(vars)) else data
 }
 
 is_mapbox <- function(p) {
@@ -146,6 +155,41 @@ is_type <- function(p, type) {
   all(types %in% type)
 }
 
+# Replace elements of a nested list
+# 
+# @param x a named list
+# @param indicies a vector of indices. 
+# A 1D list may be used to specify both numeric and non-numeric inidices
+# @param val the value used to 
+# @examples 
+# 
+# x <- list(a = 1)
+# # equivalent to `x$a <- 2`
+# re_place(x, "a", 2)
+# 
+# y <- list(a = list(list(b = 2)))
+# 
+# # equivalent to `y$a[[1]]$b <- 2`
+# y <- re_place(y, list("a", 1, "b"), 3)
+# y
+
+re_place <- function(x, indicies = 1, val) {
+  
+  expr <- call("[[", quote(x), indicies[[1]])
+  if (length(indicies) == 1) {
+    eval(call("<-", expr, val))
+    return(x)
+  }
+  
+  for (i in seq(2, length(indicies))) {
+    expr <- call("[[", expr, indicies[[i]])
+  }
+  
+  eval(call("<-", expr, val))
+  x
+}
+
+
 # retrive mapbox token if one is set; otherwise, throw error
 mapbox_token <- function() {
   token <- Sys.getenv("MAPBOX_TOKEN", NA)
@@ -161,6 +205,95 @@ mapbox_token <- function() {
   token
 }
 
+fit_bounds <- function(p) {
+  # Compute layout.mapboxid._fitBounds, an internal attr that has special client-side logic
+  # PS. how the hell does mapbox not have a way to set initial map bounds?
+  # https://github.com/mapbox/mapbox-gl-js/issues/1970
+  mapboxIDs <- grep("^mapbox", sapply(p$x$data, "[[", "subplot"), value = TRUE)
+  for (id in mapboxIDs) {
+    bboxes <- lapply(p$x$data, function(tr) if (identical(id, tr$subplot)) tr[["_bbox"]])
+    rng <- bboxes2range(bboxes, f = 0.01)
+    if (!length(rng)) next
+    # intentionally an array of numbers in [west, south, east, north] order
+    # https://www.mapbox.com/mapbox-gl-js/api/#lnglatboundslike
+    p$x$layout[[id]]$`_fitBounds` <- list(
+      bounds = c(
+        min(rng$xrng),
+        min(rng$yrng),
+        max(rng$xrng),
+        max(rng$yrng)
+      ),
+      options = list(
+        padding = 10, 
+        linear = FALSE,
+        # NOTE TO SELF: can do something like this to customize easing
+        # easing = htmlwidgets::JS("function(x) { return 1; }"),
+        offset = c(0, 0)
+      )
+    )
+    p$x$layout[[id]]$center$lat <- mean(rng$yrng)
+    p$x$layout[[id]]$center$lon <- mean(rng$xrng)
+  }
+  
+  # Compute layout.geoid.lonaxis.range & layout.geoid.lataxis.range
+  # for scattergeo
+  geoIDs <- grep("^geo", sapply(p$x$data, "[[", "geo"), value = TRUE)
+  for (id in geoIDs) {
+    bboxes <- lapply(p$x$data, function(tr) if (identical(id, tr$geo)) tr[["_bbox"]])
+    rng <- bboxes2range(bboxes, f = 0.01)
+    if (!length(rng)) next
+    p$x$layout[[id]]$lataxis$range <- rng$yrng
+    p$x$layout[[id]]$lonaxis$range <- rng$xrng
+  }
+  
+  # Compute layout.axisid.scaleanchor & layout.axisid.scaleratio
+  # for scatter/scattergl
+  rows <- compact(lapply(p$x$data, function(x) c(x[["xaxis"]], x[["yaxis"]])))
+  for (i in seq_along(rows)) {
+    xid <- rows[[i]][[1]]
+    yid <- rows[[i]][[2]]
+    bboxes <- lapply(p$x$data, function(tr) {
+      if (identical(xid, tr$xaxis) && identical(yid, tr$yaxis)) tr[["_bbox"]]
+    })
+    rng <- bboxes2range(bboxes, f = 0.01)
+    if (!length(rng)) next
+    xname <- sub("x", "xaxis", xid)
+    yname <- sub("y", "yaxis", yid)
+    # default to empty axes
+    # TODO: is there a set of projections where it makes sense to show a cartesian grid?
+    eaxis <- list(showgrid = FALSE, zeroline = FALSE, ticks = "", showticklabels = FALSE)
+    p$x$layout[[xname]] <- modify_list(eaxis, p$x$layout[[xname]])
+    p$x$layout[[yname]] <- modify_list(eaxis, p$x$layout[[yname]])
+    # remove default axis titles
+    p$x$layout[[xname]]$title <- p$x$layout[[xname]]$title %|D|% NULL
+    p$x$layout[[yname]]$title <- p$x$layout[[yname]]$title %|D|% NULL
+    p$x$layout[[xname]]$scaleanchor <- yid
+    # TODO: only do this for lat/lon dat
+    p$x$layout[[xname]]$scaleratio <- cos(mean(rng$yrng) * pi/180)
+  }
+  
+  # Internal _bbox field no longer needed
+  #p$x$data <- lapply(p$x$data, function(tr) { tr[["_bbox"]] <- NULL; tr })
+  p
+}
+
+# find the x/y layout range of a collection of trace._bboxes
+bboxes2range <- function(bboxes, ...) {
+  if (sum(lengths(bboxes)) == 0) return(NULL)
+  yrng <- c(
+    min(unlist(lapply(bboxes, "[[", "ymin")), na.rm = TRUE),
+    max(unlist(lapply(bboxes, "[[", "ymax")), na.rm = TRUE)
+  )
+  xrng <- c(
+    min(unlist(lapply(bboxes, "[[", "xmin")), na.rm = TRUE),
+    max(unlist(lapply(bboxes, "[[", "xmax")), na.rm = TRUE)
+  )
+  list(
+    yrng = grDevices::extendrange(yrng, ...),
+    xrng = grDevices::extendrange(xrng, ...)
+  )
+}
+
 # rename attrs (unevaluated arguments) from geo locations (lat/lon) to cartesian
 geo2cartesian <- function(p) {
   p$x$attrs <- lapply(p$x$attrs, function(tr) {
@@ -171,6 +304,19 @@ geo2cartesian <- function(p) {
   p
 }
 
+cartesian2geo <- function(p) {
+  p$x$data <- lapply(p$x$data, function(tr) {
+    if (isTRUE(tr[["type"]] %in% c("scattermapbox", "scattergeo"))) {
+      tr[["lat"]] <- tr[["lat"]] %||% tr[["y"]]
+      tr[["lon"]] <- tr[["lon"]] %||% tr[["x"]]
+      tr[c("x", "y")] <- NULL
+    }
+    tr
+  })
+  p
+}
+
+
 is_subplot <- function(p) {
   isTRUE(p$x$subplot)
 }
@@ -189,6 +335,9 @@ supply_defaults <- function(p) {
     }
     tr
   })
+  # hack to avoid https://github.com/ropensci/plotly/issues/945
+  if (is_type(p, "parcoords")) p$x$layout$margin$t <- NULL
+  
   # supply domain defaults
   geoDomain <- list(x = c(0, 1), y = c(0, 1))
   if (is_geo(p) || is_mapbox(p)) {
@@ -196,19 +345,14 @@ supply_defaults <- function(p) {
     p$x$layout[[p$x$layout$mapType]] <- modify_list(
       list(domain = geoDomain), p$x$layout[[p$x$layout$mapType]]
     )
-  } else {
-    axes <- if (is_type(p, "scatterternary"))  {
-      c("aaxis", "baxis", "caxis") 
-    } else if (is_type(p, "pie") || is_type(p, "parcoords")) {
-      # hack to avoid https://github.com/ropensci/plotly/issues/945
-      p$x$layout$margin <- NULL
-      NULL
-    } else {
-      c("xaxis", "yaxis")
-    }
+  } else if (!length(p$x$layout[["grid"]])) {
+    types <- vapply(p$x$data, function(tr) tr[["type"]] %||% "scatter", character(1))
+    axes <- unlist(lapply(types, function(x) {
+      grep("^[a-z]axis$", names(Schema$traces[[x]]$attributes), value = TRUE) %||% NULL
+    }))
     for (axis in axes) {
       p$x$layout[[axis]] <- modify_list(
-        list(domain = c(0, 1)), p$x$layout[[axis]]
+        list(domain = c(0, 1), automargin = TRUE), p$x$layout[[axis]]
       )
     }
   }
@@ -218,12 +362,6 @@ supply_defaults <- function(p) {
 supply_highlight_attrs <- function(p) {
   # set "global" options via crosstalk variable
   p$x$highlight <- p$x$highlight %||% highlight_defaults()
-  p <- htmlwidgets::onRender(
-    p, sprintf(
-      "function(el, x) { var ctConfig = crosstalk.var('plotlyCrosstalkOpts').set(%s); }", 
-      to_JSON(p$x$highlight)
-    )
-  )
   
   # defaults are now populated, allowing us to populate some other 
   # attributes such as the selectize widget definition
@@ -241,7 +379,9 @@ supply_highlight_attrs <- function(p) {
 
     # include one selectize dropdown per "valid" SharedData layer
     if (isTRUE(p$x$highlight$selectize)) {
-      p$x$selectize[[new_id()]] <- list(
+      # Hash i (the crosstalk group id) so that it can be used
+      # as an HTML id client-side (i.e., key shouldn't contain spaces)
+      p$x$selectize[[rlang::hash(i)]] <- list(
         items = data.frame(value = k, label = k), group = i
       )
     }
@@ -252,7 +392,7 @@ supply_highlight_attrs <- function(p) {
       p <- htmlwidgets::onRender(
         p, sprintf(
           "function(el, x) { crosstalk.group('%s').var('selection').set(%s) }", 
-          i, jsonlite::toJSON(vals, auto_unbox = FALSE)
+          i, jsonlite::toJSON(as.character(vals), auto_unbox = FALSE)
         )
       )
     }
@@ -284,13 +424,18 @@ verify_attr_names <- function(p) {
     c(names(Schema$layout$layoutAttributes), c("barmode", "bargap", "mapType")),
     "layout"
   )
+  attrs_name_check(
+    names(p$x$config),
+    names(Schema$config),
+    "config"
+  )
   for (tr in seq_along(p$x$data)) {
     thisTrace <- p$x$data[[tr]]
     attrSpec <- Schema$traces[[thisTrace$type %||% "scatter"]]$attributes
     # make sure attribute names are valid
     attrs_name_check(
       names(thisTrace), 
-      c(names(attrSpec), "key", "set", "frame", "transforms", "_isNestedKey", "_isSimpleKey"), 
+      c(names(attrSpec), "key", "set", "frame", "transforms", "_isNestedKey", "_isSimpleKey", "_isGraticule", "_bbox"), 
       thisTrace$type
     )
   }
@@ -302,12 +447,9 @@ verify_attr_names <- function(p) {
 # ensure both the layout and trace attributes adhere to the plot schema
 verify_attr_spec <- function(p) {
   if (!is.null(p$x$layout)) {
-    layoutNames <- names(p$x$layout)
-    layoutNew <- verify_attr(
-      setNames(p$x$layout, sub("[0-9]+$", "", layoutNames)),
-      Schema$layout$layoutAttributes
+    p$x$layout <- verify_attr(
+      p$x$layout, Schema$layout$layoutAttributes, layoutAttr = TRUE
     )
-    p$x$layout <- setNames(layoutNew, layoutNames)
   }
   for (tr in seq_along(p$x$data)) {
     thisTrace <- p$x$data[[tr]]
@@ -321,57 +463,79 @@ verify_attr_spec <- function(p) {
   p
 }
 
-verify_attr <- function(proposed, schema) {
+verify_attr <- function(proposed, schema, layoutAttr = FALSE) {
   for (attr in names(proposed)) {
-    attrSchema <- schema[[attr]]
+    attrSchema <- schema[[attr]] %||% schema[[sub("[0-9]+$", "", attr)]]
     # if schema is missing (i.e., this is an un-official attr), move along
     if (is.null(attrSchema)) next
+    
     valType <- tryNULL(attrSchema[["valType"]]) %||% ""
     role <- tryNULL(attrSchema[["role"]]) %||% ""
     arrayOK <- tryNULL(attrSchema[["arrayOk"]]) %||% FALSE
+    isDataArray <- identical(valType, "data_array")
     
     # where applicable, reduce single valued vectors to a constant 
-    # (while preserving any 'special' attribute class)
-    if (!identical(valType, "data_array") && !arrayOK && !identical(role, "object")) {
-      proposed[[attr]] <- structure(
-        unique(proposed[[attr]]), 
-        class = oldClass(proposed[[attr]])
-      )
+    # (while preserving attributes)
+    if (!isDataArray && !arrayOK && !identical(role, "object")) {
+      proposed[[attr]] <- retain(proposed[[attr]], uniq)
+    }
+    
+    # If we deliberately only want hover on fills, send a string to 
+    # plotly.js so it does something sensible 
+    if (identical(proposed[["hoveron"]], "fills")) {
+      proposed[["text"]] <- paste(uniq(proposed[["text"]]), collapse = "\n")
     }
     
     # ensure data_arrays of length 1 are boxed up by to_JSON()
-    if (identical(valType, "data_array")) {
+    if (isDataArray) {
       proposed[[attr]] <- i(proposed[[attr]])
     }
     
-    # do the same for "sub-attributes"
-    # TODO: should this be done recursively?
-    if (identical(role, "object")) {
-      for (attr2 in names(proposed[[attr]])) {
-        if (is.null(attrSchema[[attr2]])) next
-        valType2 <- tryNULL(attrSchema[[attr2]][["valType"]]) %||% ""
-        role2 <- tryNULL(attrSchema[[attr2]][["role"]]) %||% ""
-        arrayOK2 <- tryNULL(attrSchema[[attr2]][["arrayOk"]]) %||% FALSE
-        
-        if (!identical(valType2, "data_array") && !arrayOK2 && !identical(role2, "object")) {
-          proposed[[attr]][[attr2]] <- structure(
-            unique(proposed[[attr]][[attr2]]), 
-            class = oldClass(proposed[[attr]][[attr2]])
-          )
-        }
-        
-        # ensure data_arrays of length 1 are boxed up by to_JSON()
-        if (identical(valType2, "data_array")) {
-          proposed[[attr]][[attr2]] <- i(proposed[[attr]][[attr2]])
-        }
+    # tag 'src-able' attributes (needed for api_create())
+    # note that layout has 'src-able' attributes that shouldn't
+    # be turned into grids https://github.com/ropensci/plotly/pull/1489
+    isSrcAble <- !is.null(schema[[paste0(attr, "src")]]) && length(proposed[[attr]]) > 1
+    if ((isDataArray || isSrcAble) && !isTRUE(layoutAttr)) {
+      proposed[[attr]] <- structure(proposed[[attr]], apiSrc = TRUE)
+    }
+    
+    if (length(proposed[["name"]]) > 0) {
+      proposed$name <- uniq(proposed$name)
+    }
+    
+    # if marker.size was populated via `size` arg (i.e., internal map_size()), 
+    # then it should _always_ be an array
+    # of appropriate length...
+    # (when marker.size is a constant, it always sets the diameter!)
+    # https://codepen.io/cpsievert/pen/zazXgw
+    # https://github.com/plotly/plotly.js/issues/2735
+    if (is.default(proposed$marker$size)) {
+      s <- proposed$marker[["size"]]
+      if (length(s) == 1) {
+        # marker.size could be of length 1, but we may have multiple 
+        # markers -- in that case, if marker.size is an array 
+        # of length 1 will result in just one marker
+        # https://codepen.io/cpsievert/pen/aMmOza
+        n <- length(proposed[["x"]] %||% proposed[["y"]] %||% proposed[["lat"]] %||% proposed[["lon"]])
+        proposed$marker[["size"]] <- default(i(rep(s, n)))
       }
     }
+    
+    # do the same for "sub-attributes"
+    if (identical(role, "object") && is.recursive(proposed[[attr]])) {
+      proposed[[attr]] <- verify_attr(proposed[[attr]], attrSchema, layoutAttr = layoutAttr)
+    }
   }
+  
   proposed
 }
 
 attrs_name_check <- function(proposedAttrs, validAttrs, type = "scatter") {
   illegalAttrs <- setdiff(proposedAttrs, validAttrs)
+  if ("titlefont" %in% illegalAttrs) {
+    warning("The titlefont attribute is deprecated. Use title = list(font = ...) instead.", call. = FALSE)
+    illegalAttrs <- setdiff(illegalAttrs, "titlefont")
+  }
   if (length(illegalAttrs)) {
     warning("'", type, "' objects don't have these attributes: '",
             paste(illegalAttrs, collapse = "', '"), "'\n", 
@@ -423,7 +587,7 @@ verify_type <- function(trace) {
     message(
       "No ", trace$type, " mode specifed:\n",
       "  Setting the mode to markers\n",
-      "  Read more about this attribute -> https://plot.ly/r/reference/#scatter-mode"
+      "  Read more about this attribute -> https://plotly.com/r/reference/#scatter-mode"
     )
     trace$mode <- "markers"
   }
@@ -434,7 +598,7 @@ relay_type <- function(type) {
   message(
     "No trace type specified:\n", 
     "  Based on info supplied, a '", type, "' trace seems appropriate.\n",
-    "  Read more about this trace type -> https://plot.ly/r/reference/#", type
+    "  Read more about this trace type -> https://plotly.com/r/reference/#", type
   )
   type
 }
@@ -447,12 +611,14 @@ translate_linebreaks <- function(p) {
     typ <- typeof(a)
     if (typ == "list") {
       # retain the class of list elements 
-      # which important for many things, such as colorbars
+      # which is important for many things, such as colorbars
       a[] <- lapply(a, recurse)
     } else if (typ == "character" && !inherits(a, "JS_EVAL")) {
       attrs <- attributes(a)
       a <- gsub("\n", br(), a, fixed = TRUE)
       attributes(a) <- attrs
+    } else if (is.factor(a)) {
+      levels(a) <- gsub("\n", br(), levels(a), fixed = TRUE)
     }
     a
   }
@@ -478,21 +644,21 @@ verify_mode <- function(p) {
   for (tr in seq_along(p$x$data)) {
     trace <- p$x$data[[tr]]
     if (grepl("scatter", trace$type %||% "scatter")) {
-      if (!is.null(trace$marker) && !grepl("markers", trace$mode %||% "")) {
+      if (user_specified(trace$marker) && !grepl("markers", trace$mode %||% "")) {
         message(
           "A marker object has been specified, but markers is not in the mode\n",
           "Adding markers to the mode..."
         )
         p$x$data[[tr]]$mode <- paste0(p$x$data[[tr]]$mode, "+markers")
       }
-      if (!is.null(trace$line) && !grepl("lines", trace$mode %||% "")) {
+      if (user_specified(trace$line) && !grepl("lines", trace$mode %||% "")) {
         message(
           "A line object has been specified, but lines is not in the mode\n",
           "Adding lines to the mode..."
         )
         p$x$data[[tr]]$mode <- paste0(p$x$data[[tr]]$mode, "+lines")
       }
-      if (!is.null(trace$textfont) && !grepl("text", trace$mode %||% "")) {
+      if (user_specified(trace$textfont) && !grepl("text", trace$mode %||% "")) {
         warning(
           "A textfont object has been specified, but text is not in the mode\n",
           "Adding text to the mode..."
@@ -504,6 +670,57 @@ verify_mode <- function(p) {
   p
 }
 
+
+verify_colorscale <- function(p) {
+  p$x$data <- lapply(p$x$data, function(trace) {
+    trace$colorscale <- colorscale_json(trace$colorscale)
+    trace$marker$colorscale <- colorscale_json(trace$marker$colorscale)
+    trace
+  }) 
+  p
+}
+
+# Coerce `x` into a data structure that can map to a colorscale attribute.
+# Note that colorscales can either be the name of a scale (e.g., 'Rainbow') or 
+# a 2D array (e.g., [[0, 'rgb(0,0,255)'], [1, 'rgb(255,0,0)']])
+colorscale_json <- function(x) {
+  if (!length(x)) return(x)
+  if (is.character(x)) return(x)
+  if (is.matrix(x)) {
+    if (ncol(x) != 2) stop("A colorscale matrix requires two columns")
+    x <- as.data.frame(x)
+    x[, 1] <- as.numeric(x[, 1])
+  }
+  # ensure a list like this: list(list(0, 0.5, 1), list("red", "white", "blue"))
+  # converts to the correct dimensions: [[0, 'red'], [0.5, 'white'], [1, 'blue']]
+  if (is.list(x) && length(x) == 2) {
+    n1 <- length(x[[1]])
+    n2 <- length(x[[2]])
+    if (n1 != n2 || n1 == 0 || n2 == 0) {
+      warning("A colorscale list must of elements of the same (non-zero) length")
+    } else if (!is.data.frame(x) && can_be_numeric(x[[1]])) {
+      x <- data.frame(
+        val = as.numeric(x[[1]]),
+        col = as.character(x[[2]]),
+        stringsAsFactors = FALSE
+      )
+      x <- setNames(x, NULL)
+    }
+  }
+  x
+}
+
+can_be_numeric <- function(x) {
+  xnum <- suppressWarnings(as.numeric(x))
+  sum(is.na(x)) == sum(is.na(xnum))
+}
+
+# if an object (e.g. trace.marker) contains a non-default attribute, it has been user-specified
+user_specified <- function(obj = NULL) {
+  if (!length(obj)) return(FALSE)
+  !all(rapply(obj, is.default))
+}
+
 # populate categorical axes using categoryorder="array" & categoryarray=[]
 populate_categorical_axes <- function(p) {
   axes <- p$x$layout[grepl("^xaxis|^yaxis", names(p$x$layout))] %||%
@@ -574,7 +791,15 @@ verify_key_type <- function(p) {
   for (i in seq_along(keys)) {
     k <- keys[[i]]
     if (is.null(k)) next
-    uk <- unique(k)
+    if ("select" %in% p$x$layout$clickmode && "plotly_click" %in% p$x$highlight$on) {
+      warning(
+        "`layout.clickmode` = 'select' is not designed to work well with ",
+        "the R package's linking framework (i.e. crosstalk support).",
+        call. = FALSE
+      )
+    }
+    # does it *ever* make sense to have a missing key value?
+    uk <- uniq(k)
     if (length(uk) == 1) {
       # i.e., the key for this trace has one value. In this case, 
       # we don't have iterate through the entire key, so instead, 
@@ -602,14 +827,15 @@ verify_webgl <- function(p) {
     return(p)
   }
   types <- sapply(p$x$data, function(x) x[["type"]][1] %||% "scatter")
-  idx <- paste0(types, "gl") %in% names(Schema$traces)
-  if (any(!idx)) {
+  can_gl <- paste0(types, "gl") %in% names(Schema$traces)
+  already_gl <- grepl("gl$", types)
+  if (any(!can_gl & !already_gl)) {
     warning(
       "The following traces don't have a WebGL equivalent: ",
-      paste(which(!idx), collapse = ", ")
+      paste(which(!can_gl & !already_gl), collapse = ", ")
     )
   }
-  for (i in which(idx)) {
+  for (i in which(can_gl)) {
     p$x$data[[i]]$type <- paste0(p$x$data[[i]]$type, "gl")
   }
   p
@@ -626,8 +852,9 @@ verify_showlegend <- function(p) {
   }
   show <- vapply(p$x$data, function(x) x$showlegend %||% TRUE, logical(1))
   # respect only _user-specified_ defaults 
+  isSinglePie <- identical("pie", unlist(lapply(p$x$data, function(tr) tr$type))) 
   p$x$layout$showlegend <- p$x$layout$showlegend %|D|%
-    default(sum(show) > 1 || isTRUE(p$x$highlight$showInLegend))
+    default(sum(show) > 1 || isTRUE(p$x$highlight$showInLegend) || isSinglePie)
   p
 }
 
@@ -639,7 +866,7 @@ verify_guides <- function(p) {
   }
   
   isVisibleBar <- function(tr) {
-    is.colorbar(tr) && isTRUE(tr$showscale %||% TRUE)
+    is.colorbar(tr) && (tr$showscale %||% TRUE)
   }
   isBar <- vapply(p$x$data, isVisibleBar, logical(1))
   nGuides <- sum(isBar) + has_legend(p)
@@ -647,17 +874,31 @@ verify_guides <- function(p) {
   if (nGuides > 1) {
     
     # place legend at bottom since its scrolly
-    p$x$layout$legend <- modify_list(
-      list(y = 1 - ((nGuides - 1) / nGuides), yanchor = "top"),
-      p$x$layout$legend
-    )
+    yanchor <- default("top")
+    y <- default(1 - ((nGuides - 1) / nGuides))
+    p$x$layout$legend$yanchor <- p$x$layout$legend$yanchor %|D|% yanchor
+    p$x$layout$legend$y <- p$x$layout$legend[["y"]] %|D|% y
     
+    # shrink/position colorbars
     idx <- which(isBar)
     for (i in seq_along(idx)) {
-      p <- colorbar_built(
-        p, which = i, len = 1 / nGuides, y = 1 - ((i - 1) / nGuides), 
-        lenmode = "fraction",  yanchor = "top"
-      )
+      len     <- default(1 / nGuides)
+      lenmode <- default("fraction")
+      y       <- default(1 - ((i - 1) / nGuides))
+      
+      j <- idx[[i]]
+      tr <- p$x$data[[j]]
+      if (inherits(tr, "zcolor")) {
+        p$x$data[[j]]$colorbar$len <- tr$colorbar$len %|D|% len
+        p$x$data[[j]]$colorbar$lenmode <- tr$colorbar$lenmode %|D|% lenmode
+        p$x$data[[j]]$colorbar$y <- tr$colorbar$y %|D|% y
+        p$x$data[[j]]$colorbar$yanchor <- tr$colorbar$yanchor %|D|% yanchor
+      } else {
+        p$x$data[[j]]$marker$colorbar$len <- tr$marker$colorbar$len %|D|% len
+        p$x$data[[j]]$marker$colorbar$lenmode <- tr$marker$colorbar$lenmode %|D|% lenmode
+        p$x$data[[j]]$marker$colorbar$y <- tr$marker$colorbar$y %|D|% y
+        p$x$data[[j]]$marker$colorbar$yanchor <- tr$marker$colorbar$yanchor %|D|% yanchor
+      }
     }
     
   }
@@ -665,6 +906,39 @@ verify_guides <- function(p) {
   p
 }
 
+verify_mathjax <- function(p) {
+  hasMathjax <- "mathjax" %in% sapply(p$dependencies, "[[", "name")
+  if (hasMathjax) return(p)
+  
+  hasTeX <- any(rapply(p$x, is.TeX))
+  if (!hasTeX) return(p)
+  
+  # TODO: it would be much better to add the dependency here, but
+  # htmlwidgets doesn't currently support adding dependencies at print-time!
+  warning(
+    "Detected the use of `TeX()`, but mathjax has not been specified. ",
+    "Try running `config(.Last.value, mathjax = 'cdn')`",
+    call. = FALSE
+  )
+  p
+}
+
+verify_scattergl_platform <- function(p) {
+  if (!identical(.Platform$OS.type, "windows")) return(p)
+  if (!is_rstudio()) return(p)
+  
+  types <- vapply(p$x$data, function(x) x[["type"]] %||% "scatter", character(1))
+  if ("scattergl" %in% types) {
+    warning(
+      "'scattergl' trace types don't currently render in RStudio on Windows. ",
+      "Open in another web browser (IE, Chrome, Firefox, etc).",
+      call. = FALSE
+    )
+  }
+  
+  p
+}
+
 has_marker <- function(types, modes) {
   is_scatter <- grepl("scatter", types)
   ifelse(is_scatter, grepl("marker", modes), has_attr(types, "marker"))
@@ -680,6 +954,10 @@ has_text <- function(types, modes) {
   ifelse(is_scatter, grepl("text", modes), has_attr(types, "textfont"))
 }
 
+has_color_array <- function(types, mode = "marker") {
+  vapply(types, function(x) isTRUE(tryNULL(Schema$traces[[x]]$attributes[[mode]]$color$arrayOk)), logical(1))
+}
+
 has_attr <- function(types, attr = "marker") {
   if (length(attr) != 1) stop("attr must be of length 1")
   vapply(types, function(x) attr %in% names(Schema$traces[[x]]$attributes), logical(1))
@@ -739,18 +1017,19 @@ grab <- function(what = "username") {
 
 # try to grab an object key from a JSON file (returns empty string on error)
 try_file <- function(f, what) {
-  tryCatch(jsonlite::fromJSON(f)[[what]], error = function(e) NULL)
+  tryCatch(jsonlite::read_json(f)[[what]], error = function(e) NULL)
 }
 
 # preferred defaults for toJSON mapping
 to_JSON <- function(x, ...) {
   jsonlite::toJSON(x, digits = 50, auto_unbox = TRUE, force = TRUE,
-                   null = "null", na = "null", ...)
+                   null = "null", na = "null", 
+                   time_format = "%Y-%m-%d %H:%M:%OS6",  ...)
 }
 
 # preferred defaults for toJSON mapping
 from_JSON <- function(x, ...) {
-  jsonlite::fromJSON(x, simplifyDataFrame = FALSE, simplifyMatrix = FALSE, ...)
+  jsonlite::parse_json(x, simplifyVector = TRUE, simplifyDataFrame = FALSE, simplifyMatrix = FALSE, ...)
 }
 
 i <- function(x) {
@@ -798,7 +1077,9 @@ remove_class <- function(x, y) {
 # TODO: what are some other common configuration options we want to support??
 get_domain <- function(type = "") {
   if (type == "api") {
-    Sys.getenv("plotly_api_domain", "https://api.plot.ly")
+    # new onprem instances don't have an https://api-thiscompany.plot.ly
+    # but https://thiscompany.plot.ly seems to just work in that case...
+    Sys.getenv("plotly_api_domain", Sys.getenv("plotly_domain", "https://api.plot.ly"))
   } else {
     Sys.getenv("plotly_domain", "https://plot.ly")
   }
@@ -858,3 +1139,41 @@ try_library <- function(pkg, fun = NULL) {
   stop("Package `", pkg, "` required",  if (!is.null(fun)) paste0(" for `", fun, "`"), ".\n", 
        "Please install and try again.", call. = FALSE)
 }
+
+# a la shiny:::is_available
+is_available <- function(package, version = NULL) {
+  installed <- nzchar(system.file(package = package))
+  if (is.null(version)) {
+    return(installed)
+  }
+  installed && isTRUE(utils::packageVersion(package) >= version)
+}
+
+# similar logic to rstudioapi::isAvailable()
+is_rstudio <- function() {
+  identical(.Platform$GUI, "RStudio")
+}
+
+# nchar() needs a non-empty character vector; sometimes x will be a
+# factor, or an empty vector.
+robust_nchar <- function(x, ...) {
+  if (length(x)) nchar(as.character(x), ...)
+  else 0
+}
+
+# Extract longest element, or blank if none
+longest_element <- function(x) {
+  if (length(x))
+    x[which.max(robust_nchar(x))]
+  else
+    ""
+}
+
+# A dplyr::group_by wrapper for the add argument
+group_by_add <- function(..., add = TRUE) {
+  if (packageVersion('dplyr') >= '1.0') {
+    dplyr::group_by(...,  .add = add)
+  } else {
+    dplyr::group_by(...,  add = add)
+  } 
+}
