@@ -1,18 +1,23 @@
 context("highlighting and animation")
 
 m <- crosstalk::SharedData$new(mtcars, ~vs)
+p <- plot_ly(m, x = ~wt, y = ~mpg) %>% add_markers()
 
 test_that("SharedData produces key/set in plot_ly", {
-  m <- crosstalk::SharedData$new(mtcars, ~vs)
-  p <- plot_ly(m, x = ~wt, y = ~mpg) %>% add_markers()
   tr <- plotly_build(p)$x$data[[1]]
-  
   expect_true(all(tr$key == m$key()))
   expect_identical(tr$set, m$groupName())
   expect_false(tr$`_isNestedKey` %||% FALSE)
   expect_false(tr$`_isSimpleKey` %||% FALSE)
 })
 
+test_that("Warning is thrown when clickmode='select' is used with crosstalk", {
+  expect_warning(
+    plotly_build(layout(p, clickmode = "select")),
+    "not designed to work well"
+  )
+})
+
 test_that("SharedData produces key/set in ggplotly", {
   p <- ggplot(m, aes(x = wt, y = mpg)) + geom_point()
   tr <- plotly_build(p)$x$data[[1]]
@@ -24,6 +29,8 @@ test_that("SharedData produces key/set in ggplotly", {
   expect_false(tr$`_isSimpleKey` %||% FALSE)
 })
 
+
+
 test_that("crosstalk keys are inherited in a layer with inherit = FALSE", {
   
   p <- txhousing %>%
@@ -63,6 +70,18 @@ test_that("Simple scatterplot brushing with plot_ly() and subplot()", {
 })
 
 
+test_that("group_by.plotly() retains crosstalk set", {
+  b <- mtcars %>%
+    crosstalk::SharedData$new(group = "foo") %>%
+    plot_ly(x = ~mpg, y = ~hp) %>%
+    group_by(am) %>%
+    add_markers() %>%
+    plotly_build()
+  expect_equal(b$x$data[[1]]$set, "foo")
+  expect_true(all(b$x$data[[1]]$key == row.names(mtcars)))
+})
+
+
 
 # Ignore for now https://github.com/ggobi/ggally/issues/264
 #test_that("SharedData produces key/set in ggpairs", {
@@ -340,8 +359,8 @@ test_that("simple animation targeting works", {
 
 test_that("animation frames are boxed up correctly", {
   dallas <- subset(txhousing, city == "Dallas" & month == 1)
-  p <- ggplot(dallas) +
-    geom_point(aes(x = volume, y = sales, frame = year))
+  p <- ggplot(dallas, aes(x = volume, y = sales, frame = year)) +
+    geom_point()
   l <- plotly_build(p)$x
   
   for (i in seq_along(l$frames)) {
@@ -372,6 +391,7 @@ test_that("animation button can be customized", {
 
 test_that("sf works with crosstalk", {
   skip_if_not_installed("sf")
+  skip_if_not_installed("s2")
   
   nc <- sf::st_read(system.file("shape/nc.shp", package = "sf"), quiet = TRUE)
   # shared data will make the polygons "query-able"
