@@ -4,7 +4,7 @@ context("Area")
 # Expect traces function
 expect_traces <- function(gg, n_traces, name) {
   stopifnot(is.numeric(n_traces))
-  save_outputs(gg, paste0("area-", name))
+  expect_doppelganger_built(gg, paste0("area-", name))
   L <- gg2list(gg)
   all_traces <- L$data
   no_data <- sapply(all_traces, function(tr) {
