@@ -10,15 +10,16 @@ test_that("Basic geom_quantile() works", {
     geom_point() +
     geom_quantile()
   
-  l <- plotly_build(p)$x
+  # partial match of 'coef' to 'coefficients' 
+  l <- suppressWarnings(plotly_build(p)$x)
   
   expect_length(l$data, 4)
   
   for (i in 2:4) {
     tr <- l$data[[i]]
-    expect_equal(tr$type, "scatter")
-    expect_equal(tr$mode, "lines")
-    expect_equal(
+    expect_equivalent(tr$type, "scatter")
+    expect_equivalent(tr$mode, "lines")
+    expect_equivalent(
       tr$line$color, toRGB(GeomQuantile$default_aes[["colour"]])
     )
   }
@@ -35,15 +36,16 @@ test_that("Can specify gpar() in geom_quantile()", {
     geom_point() +
     geom_quantile(colour = "red", alpha = 0.5)
   
-  l <- plotly_build(p)$x
+  # partial match of 'coef' to 'coefficients' 
+  l <- suppressWarnings(plotly_build(p)$x)
   
   expect_length(l$data, 4)
   
   for (i in 2:4) {
     tr <- l$data[[i]]
-    expect_equal(tr$type, "scatter")
-    expect_equal(tr$mode, "lines")
-    expect_equal(
+    expect_equivalent(tr$type, "scatter")
+    expect_equivalent(tr$mode, "lines")
+    expect_equivalent(
       tr$line$color, toRGB("red", 0.5)
     )
   }
