@@ -1,7 +1,7 @@
 context("plotly data")
 
 test_that("uniq works as expected", {
-  expect_equal(uniq(c("red", "red", NA)), "red")
+  expect_equivalent(uniq(c("red", "red", NA)), "red")
 })
 
 test_that("plotly_data returns empty data frame when none is specified", {
@@ -11,11 +11,11 @@ test_that("plotly_data returns empty data frame when none is specified", {
 
 test_that("plotly_data returns data frame", {
   d <- plotly_data(plot_ly(economics))
-  expect_identical(economics, d)
+  expect_identical(as.data.frame(economics), as.data.frame(d))
 })
 
 test_that("plotly_data preserves groups in data", {
-  d <- plotly_data(group_by_(plot_ly(mtcars), c("vs", "am")))
+  d <- plotly_data(group_by(plot_ly(mtcars), vs, am))
   expect_true(dplyr::groups(d)[[1]] == "vs")
 })
 
@@ -29,16 +29,16 @@ test_that("add_fun can apply two different chunks of data to a plot", {
       p %>% slice(which.min(mpg)) %>% add_annotations(text = "Bad mileage")
     })
   l <- plotly_build(p)[["x"]]
-  expect_equal(length(l$layout$annotations), 2)
-  expect_equal(
+  expect_equivalent(length(l$layout$annotations), 2)
+  expect_equivalent(
     sort(sapply(l$layout$annotations, "[[", "text")),
     c("Bad mileage", "Good mileage")
   )
-  expect_equal(
+  expect_equivalent(
     sort(sapply(l$layout$annotations, "[[", "x")),
     c(1.835, 5.250)
   )
-  expect_equal(
+  expect_equivalent(
     sort(sapply(l$layout$annotations, "[[", "y")),
     c(10.4, 33.9)
   )
