@@ -2,7 +2,7 @@ context("Filename")
 
 test_that("filename supports names with paths included ", {
   skip_on_cran()
-  skip_on_pull_request()
+  skip_if_not_master()
   p <- plot_ly(mtcars, x = ~wt, y = ~vs)
   filename <- "directory/awesome"
   # trash the file if it already exists
@@ -11,7 +11,7 @@ test_that("filename supports names with paths included ", {
     endpt <- sprintf("files/%s/trash", file$fid)
     res <- api(endpt, "POST")
   }
-  f <- plotly_POST(p, filename = filename)
+  f <- api_create(p, filename = filename)
   expect_match(f$filename, "awesome")
   expect_true(f$parented)
 })
