@@ -1,31 +1,31 @@
-context("get_figure")
+context("api_download_plot")
 
 test_that("requests made by a user who doesn't exist error a 404", {
   skip_on_cran()
-  skip_on_pull_request()
+  skip_if_not_master()
   expect_error({
-    get_figure("klmadslfjdfljdsf", 0)
+    api_download_plot(0, "klmadslfjdfljdsf")
   }, ".*404.*")
 })
 
 test_that("requests made to retrieve a figure that doesn't exist returns a 404", {
   skip_on_cran()
-  skip_on_pull_request()
+  skip_if_not_master()
   expect_error({
-    get_figure("get_test_user", 18324823)
+    api_download_plot(18324823, "get_test_user")
   }, ".*404.*")
 })
 
 test_that("requests made to retrieve some elses private file errors", {
   skip_on_cran()
-  skip_on_pull_request()
-  expect_error(get_figure("get_test_user", 1))
+  skip_if_not_master()
+  expect_error(api_download_plot(1, "get_test_user"))
 })
 
 test_that("retrieving a public figure ... works.", {
   skip_on_cran()
-  skip_on_pull_request()
-  fig <- get_figure("get_test_user", 0)
+  skip_if_not_master()
+  fig <- api_download_plot(0, "get_test_user")
   # get the data behind the hash
   p <- plotly_build(fig)$x
   expect_equivalent(p$data[[1]]$x, c("1", "2", "3"))
@@ -33,8 +33,8 @@ test_that("retrieving a public figure ... works.", {
 
 test_that("can add traces to a subplot figure", {
   skip_on_cran()
-  skip_on_pull_request()
-  fig <- get_figure('chelsea_lyn', 6366)
+  skip_if_not_master()
+  fig <- api_download_plot(6366, 'chelsea_lyn')
   p <- add_lines(fig, x = c(1, 2, 3), y = c(4, 2, 4))
   expect_equivalent(
     length(plotly_build(fig)$x$data) + 1, 
@@ -44,8 +44,8 @@ test_that("can add traces to a subplot figure", {
 
 test_that("posting a hidden plot returns a secret key", {
   skip_on_cran()
-  skip_on_pull_request()
-  res <- plotly_POST(plot_ly(), sharing = "secret")
+  skip_if_not_master()
+  res <- api_create(plot_ly(), sharing = "secret")
   expect_true(res$share_key_enabled)
   expect_true(nchar(res$share_key) > 1)
 })
